package elasticache

var DescribeReplicationGroupsResponse = `<DescribeReplicationGroupsResponse xmlns="http://elasticache.amazonaws.com/doc/2014-07-15/">
<DescribeReplicationGroupsResult>
<ReplicationGroups>
<ReplicationGroup>
<MemberClusters>
<ClusterId>example-test-001</ClusterId>
<ClusterId>example-test-002</ClusterId>
<ClusterId>example-test-003</ClusterId>
</MemberClusters>
<NodeGroups>
<NodeGroup>
<NodeGroupId>0001</NodeGroupId>
<PrimaryEndpoint>
<Port>6379</Port>
<Address>example-test.4q8cbh.ng.0001.euw1.cache.amazonaws.com</Address>
</PrimaryEndpoint>
<Status>available</Status>
<NodeGroupMembers>
<NodeGroupMember>
<CacheClusterId>example-test-001</CacheClusterId>
<ReadEndpoint>
<Port>6379</Port>
<Address>example-test-001.4q8cbh.0001.euw1.cache.amazonaws.com</Address>
</ReadEndpoint>
<PreferredAvailabilityZone>eu-west-1a</PreferredAvailabilityZone>
<CacheNodeId>0001</CacheNodeId>
<CurrentRole>primary</CurrentRole>
</NodeGroupMember>
<NodeGroupMember>
<CacheClusterId>example-test-002</CacheClusterId>
<ReadEndpoint>
<Port>6379</Port>
<Address>example-test-002.4q8cbh.0001.euw1.cache.amazonaws.com</Address>
</ReadEndpoint>
<PreferredAvailabilityZone>eu-west-1b</PreferredAvailabilityZone>
<CacheNodeId>0001</CacheNodeId>
<CurrentRole>replica</CurrentRole>
</NodeGroupMember>
<NodeGroupMember>
<CacheClusterId>example-test-003</CacheClusterId>
<ReadEndpoint>
<Port>6379</Port>
<Address>example-test-003.4q8cbh.0001.euw1.cache.amazonaws.com</Address>
</ReadEndpoint>
<PreferredAvailabilityZone>eu-west-1c</PreferredAvailabilityZone>
<CacheNodeId>0001</CacheNodeId>
<CurrentRole>replica</CurrentRole>
</NodeGroupMember>
</NodeGroupMembers>
</NodeGroup>
</NodeGroups>
<ReplicationGroupId>example-test</ReplicationGroupId>
<Status>available</Status>
<PendingModifiedValues/>
<Description>CH Doc API test</Description>
</ReplicationGroup>
</ReplicationGroups>
</DescribeReplicationGroupsResult>
<ResponseMetadata>
<RequestId>a84997a2-8141-11e4-b193-ff2ad2b98e65</RequestId>
</ResponseMetadata>
</DescribeReplicationGroupsResponse>
`
